#!/bin/bash

# hist.sh
# Andrew Davison, ad@fivedots.coe.psu.ac.th, April 2019

: <<'COMMENT'
  Draw a live histogram of three types of network connections
  on this machine, which is updated every DELAY seconds. The
  types are specified in the connTypes[] array.

  The network data is obtained using the netstat command,
  so check it is installed.

  GnuPlot's "dumb" terminal output is used so that the
  graph can be drawn in the terminal window.

  Gnuplot also saves each plot as a PNG, and these images
  are converted to an animated GIF when this program is
  terminated.

  The generation of the GIF uses ImageMagick's convert
  so check it is installed.

  The script will keep generating plots until it is
  terminated by the user typing ctrl-C.

  The Gnuplot script waits for an integer to be piped to
  it before drawing a plot.

  If the integer is negative then the Gnuplot script will
  terminate, but that feature is not used in this example.
  Instead, Gnuplot will exit when the user types crtl-C.
COMMENT


DATAFNM="hist.dat"   # data file
PLOTFNM="hist.gp"    # Gnuplot script

let DELAY=1   # seconds

connTypes=("ESTABLISHED" "LISTEN" "TIME_WAIT")


trap ctrl_c INT

ctrl_c()
# finish by generating an animated GIF
{
  fnms="animate\_[0-9][0-9][0-9][0-9].png"

  count=$(ls -1 $fnms | wc -l)
  echo -n "Converting $count PNGs to an animated GIF..."

  convert -scale 75% -quiet $fnms  animate.gif
     # "-quiet" to ignore an occasional corrupt PNG warning
     # convert is an ImageMagick command

  rm -f $fnms  # remove the PNGs
  echo " done"
  exit
}
 


genData() 
{
  let count=0
  while true ; do
    echo -e -n "" > $1   # create / clear the data file
    # add data lines for the connection types
    for ct in ${connTypes[*]}; do
      num=$(netstat -ant | grep $ct | wc -l)
        # netstat is part of the net-tools package
      echo -e $ct"\t"$num >> $1
    done
    echo $count   # wake up gnuplot
    let count++
    sleep $DELAY
  done
}


genData $DATAFNM | gnuplot -persist -e "datafnm='${DATAFNM}'" $PLOTFNM
     # link the shell script to Gnuplot with a pipe
     # so that it can send integers to the Gnuplot script
     # to control its plotting frequency


