#!/bin/bash

# points.sh
# Andrew Davison, ad@fivedots.coe.psu.ac.th, April 2019

: <<'COMMENT'
  Draw a live plot of the number of network connections
  on this machine, which is updated every DELAY seconds.

  The network data is obtained using the netstat command,
  so check it is installed.
 
  As a new point is added, if there are already
  MAX_PTS points in the data file, then the oldest point is
  removed.
 
  The x-axis shows the minute:second when the
  data point was recorded.
 
  GnuPlot's "dumb" terminal output is used so that the
  graph can be drawn in the terminal window.

  Gnuplot also saves each plot as a PNG, and these images
  are converted to an animated GIF when this program is
  terminated.

  The generation of the GIF uses ImageMagick's convert
  so check it is installed.

  The script will keep generating plots until it is
  terminated by the user typing ctrl-C.

  The Gnuplot script waits for an integer to be piped to 
  it before drawing a plot.

  If the integer is negative then the Gnuplot script will
  terminate, but that feature is not used in this example. 
  Instead, Gnuplot will exit when the user types crtl-C.
COMMENT


DATAFNM="points.dat"   # data file 
PLOTFNM="points.gp"    # Gnuplot script

let DELAY=1   # seconds
let MAXPTS=6


trap ctrl_c INT

ctrl_c()
# finish by generating an animated GIF
{ 
  fnms="animate\_[0-9][0-9][0-9][0-9].png"

  count=$(ls -1 $fnms | wc -l)
  echo -n "Converting $count PNGs to an animated GIF..."

  convert -scale 75% -quiet $fnms animate.gif  
     # "-quiet" to ignore an occasional corrupt PNG warning
     # convert is an ImageMagick command
 
  rm -f $fnms  # remove the PNGs
  echo " done"
  exit
}


cutLine()
# remove first line if the data file has reached the
# maximum number of points (there's one data point per line)
{
  len=$(wc -l < $1)
  if [ "$len" -ge "$MAXPTS" ]
  then
    sed -i "1d" $1
  fi
}


genData() 
{
  echo -e -n "" > $1   # create / clear the data file
  while true ; do
    cutLine $1
    # add a line of data
    ts=$(date +%s)
    num=$(netstat -an | wc -l)   
        # netstat is part of the net-tools package
    echo -e $ts"\t"$num >> $1
    echo 0    # wake up gnuplot
    sleep $DELAY
  done
}


genData $DATAFNM | gnuplot -persist -e "datafnm='${DATAFNM}'" $PLOTFNM
     # link the shell script to Gnuplot with a pipe 
     # so that it can send integers to the Gnuplot script
     # to control its plotting frequency

